@@ -79,6 +79,23 @@ pub const CIRCUIT_BUFFER_SIZE: usize = 128;
 
 #[derive(Clone, Debug)]
 /// A circuit that we have constructed over the Tor network.
+///
+/// This struct is the interface used by the rest of the code, It is fairly
+/// cheaply cloneable.  None of the public methods need mutable access, since
+/// they all actually communicate with the Reactor which contains the primary
+/// mutable state, and does the actual work.
+//
+// Effectively, this struct contains two Arcs: one for `hops` and one for
+// `control` (which surely has soemthing Arc-like in it).  We cannot unify
+// these by putting a single Arc around the whole struct, and passing
+// an Arc strong reference to the `Reactor`, because then `control` would
+// not be dropped when the last user of the circuit goes away.  We could
+// make the reactor have a weak reference but weak references are more
+// expensive to dereference.
+//
+// Because of the above, cloning this struct is always going to involve
+// two atomic refcount changes/checks.  Wrapping it in another Arc would
+// be overkill.
 pub struct ClientCirc {
     /// Number of hops on this circuit.
     ///
@@ -222,7 +239,11 @@ impl ClientCirc {
         // assuming it's the last hop.
 
         let num_hops = self.hops.load(Ordering::SeqCst);
-        // FIXME(eta): could panic if num_hops is zero
+        if num_hops == 0 {
+            return Err(Error::InternalError(
+                "Number of hops specified is zero, cannot continue".into(),
+            ));
+        }
         let hop_num: HopNum = (num_hops - 1).into();
         let (sender, receiver) = mpsc::channel(STREAM_READER_BUFFER);
         let (tx, rx) = oneshot::channel();
@@ -259,11 +280,7 @@ impl ClientCirc {
 
     /// Start a DataStream (anonymized connection) to the given
     /// address and port, using a BEGIN cell.
-    async fn begin_data_stream(
-        self: Arc<Self>,
-        msg: RelayMsg,
-        optimistic: bool,
-    ) -> Result<DataStream> {
+    async fn begin_data_stream(&self, msg: RelayMsg, optimistic: bool) -> Result<DataStream> {
         let (reader, target) = self.begin_stream_impl(msg).await?;
         let mut stream = DataStream::new(reader, target);
         if !optimistic {
@@ -278,7 +295,7 @@ impl ClientCirc {
     /// The use of a string for the address is intentional: you should let
     /// the remote Tor relay do the hostname lookup for you.
     pub async fn begin_stream(
-        self: Arc<Self>,
+        &self,
         target: &str,
         port: u16,
         parameters: Option<StreamParameters>,
@@ -292,8 +309,7 @@ impl ClientCirc {
 
     /// Start a new stream to the last relay in the circuit, using
     /// a BEGIN_DIR cell.
-    // FIXME(eta): get rid of Arc here!!!
-    pub async fn begin_dir_stream(self: Arc<Self>) -> Result<DataStream> {
+    pub async fn begin_dir_stream(&self) -> Result<DataStream> {
         // Note that we always open begindir connections optimistically.
         // Since they are local to a relay that we've already authenticated
         // with and built a circuit to, there should be no additional checks
@@ -306,7 +322,7 @@ impl ClientCirc {
     ///
     /// Note that this function does not check for timeouts; that's
     /// the caller's responsibility.
-    pub async fn resolve(self: Arc<Self>, hostname: &str) -> Result<Vec<IpAddr>> {
+    pub async fn resolve(&self, hostname: &str) -> Result<Vec<IpAddr>> {
         let resolve_msg = Resolve::new(hostname);
 
         let resolved_msg = self.try_resolve(resolve_msg).await?;
@@ -327,7 +343,7 @@ impl ClientCirc {
     ///
     /// Note that this function does not check for timeouts; that's
     /// the caller's responsibility.
-    pub async fn resolve_ptr(self: Arc<Self>, addr: IpAddr) -> Result<Vec<String>> {
+    pub async fn resolve_ptr(&self, addr: IpAddr) -> Result<Vec<String>> {
         let resolve_ptr_msg = Resolve::new_reverse(&addr);
 
         let resolved_msg = self.try_resolve(resolve_ptr_msg).await?;
@@ -348,7 +364,7 @@ impl ClientCirc {
 
     /// Helper: Send the resolve message, and read resolved message from
     /// resolve stream.
-    async fn try_resolve(self: &Arc<Self>, msg: Resolve) -> Result<Resolved> {
+    async fn try_resolve(&self, msg: Resolve) -> Result<Resolved> {
         let (reader, _) = self.begin_stream_impl(msg.into()).await?;
         let mut resolve_stream = ResolveStream::new(reader);
         resolve_stream.read_msg().await
@@ -453,7 +469,7 @@ impl PendingClientCirc {
     /// There's no authentication in CRATE_FAST,
     /// so we don't need to know whom we're connecting to: we're just
     /// connecting to whichever relay the channel is for.
-    pub async fn create_firsthop_fast(self, params: CircParameters) -> Result<ClientCirc> {
+    pub async fn create_firsthop_fast(self, params: &CircParameters) -> Result<ClientCirc> {
         let (tx, rx) = oneshot::channel();
         self.circ
             .control
@@ -747,7 +763,7 @@ mod test {
             let params = CircParameters::default();
             let ret = if fast {
                 trace!("doing fast create");
-                pending.create_firsthop_fast(params).await
+                pending.create_firsthop_fast(&params).await
             } else {
                 trace!("doing ntor create");
                 pending.create_firsthop_ntor(&target, params).await
@@ -1103,7 +1119,7 @@ mod test {
             let begin_and_send_fut = async move {
                 // Here we'll say we've got a circuit, and we want to
                 // make a simple BEGINDIR request with it.
-                let mut stream = Arc::new(circ).begin_dir_stream().await.unwrap();
+                let mut stream = circ.begin_dir_stream().await.unwrap();
                 stream.write_all(b"HTTP/1.0 GET /\r\n").await.unwrap();
                 stream.flush().await.unwrap();
                 let mut buf = [0_u8; 1024];
@@ -1177,7 +1193,7 @@ mod test {
         let (chan, mut rx, sink2) = working_fake_channel(rt);
         let (circ, mut sink) = newcirc(rt, chan).await;
 
-        let circ_clone = Arc::new(circ.clone());
+        let circ_clone = circ.clone();
         let begin_and_send_fut = async move {
             // Take our circuit and make a stream on it.
             let mut stream = circ_clone
@@ -1259,11 +1275,20 @@ mod test {
                 assert_eq!(window, 1000 - 301);
                 assert_eq!(tags.len(), 3);
                 // 100
-                assert_eq!(tags[0], hex!("6400000000000000000000000000000000000000"));
+                assert_eq!(
+                    tags[0],
+                    sendme::CircTag::from(hex!("6400000000000000000000000000000000000000"))
+                );
                 // 200
-                assert_eq!(tags[1], hex!("c800000000000000000000000000000000000000"));
+                assert_eq!(
+                    tags[1],
+                    sendme::CircTag::from(hex!("c800000000000000000000000000000000000000"))
+                );
                 // 300
-                assert_eq!(tags[2], hex!("2c01000000000000000000000000000000000000"));
+                assert_eq!(
+                    tags[2],
+                    sendme::CircTag::from(hex!("2c01000000000000000000000000000000000000"))
+                );
             }
 
             let reply_with_sendme_fut = async move {
