@@ -12,6 +12,7 @@ use crate::crypto::cell::{
 use crate::util::err::ReactorError;
 use crate::{Error, Result};
 use std::collections::VecDeque;
+use std::convert::TryFrom;
 use std::marker::PhantomData;
 use std::pin::Pin;
 use tor_cell::chancell::msg::{ChanMsg, Relay};
@@ -173,6 +174,15 @@ pub(super) struct CircHop {
     outbound: VecDeque<([u8; 20], ChanCell)>,
 }
 
+/// An indicator on what we should do when we receive a cell for a circuit.
+#[derive(Copy, Clone, Debug, Eq, PartialEq)]
+enum CellStatus {
+    /// The circuit should stay open.
+    Continue,
+    /// Perform a clean shutdown on this circuit.
+    CleanShutdown,
+}
+
 impl CircHop {
     /// Create a new hop.
     pub(super) fn new(auth_sendme_optional: bool, initial_window: u16) -> Self {
@@ -309,15 +319,6 @@ where
         self.expected_hop
     }
     fn finish(&mut self, msg: RelayMsg, reactor: &mut Reactor) -> Result<()> {
-        // XXXX If two EXTEND cells are of these are launched on the
-        // same circuit at once, could they collide in this part of
-        // the function?  I don't _think_ so, but it might be a good idea
-        // to have an "extending" bit that keeps two tasks from entering
-        // extend_impl at the same time.
-        //
-        // Also we could enforce that `hop` is still what we expect it
-        // to be at this point.
-
         // Did we get the right response?
         if msg.cmd() != RelayCmd::EXTENDED2 {
             return Err(Error::CircProto(format!(
@@ -459,7 +460,7 @@ impl Reactor {
                         return Poll::Ready(Err(ReactorError::Shutdown));
                     }
                     Some(cell) => {
-                        if self.handle_cell(cx, cell)? {
+                        if self.handle_cell(cx, cell)? == CellStatus::CleanShutdown {
                             trace!("{}: reactor shutdown due to handled cell", self.unique_id);
                             return Poll::Ready(Err(ReactorError::Shutdown));
                         }
@@ -629,7 +630,7 @@ impl Reactor {
         params: &CircParameters,
     ) -> Result<()>
     where
-        L: CryptInit + ClientLayer<FWD, REV> + 'static + Send, // need all this?XXXX
+        L: CryptInit + ClientLayer<FWD, REV> + 'static + Send,
         FWD: OutboundClientLayer + 'static + Send,
         REV: InboundClientLayer + 'static + Send,
         H: ClientHandshake,
@@ -759,20 +760,22 @@ impl Reactor {
     }
 
     /// Handle a RELAY cell on this circuit with stream ID 0.
-    fn handle_meta_cell(&mut self, hopnum: HopNum, msg: RelayMsg) -> Result<()> {
+    fn handle_meta_cell(&mut self, hopnum: HopNum, msg: RelayMsg) -> Result<CellStatus> {
         // SENDME cells and TRUNCATED get handled internally by the circuit.
         if let RelayMsg::Sendme(s) = msg {
             return self.handle_sendme(hopnum, s);
         }
-        if let RelayMsg::Truncated(_) = msg {
-            // XXXX need to handle Truncated cells. This isn't the right
-            // way, but at least it's safe.
-            // TODO: If we ever do handle Truncate cells more
-            // correctly, we will need to audit all our use of HopNum
-            // to identify a layer.  Otherwise we could confuse a
-            // message from the previous hop N with a message from the
-            // new hop N.
-            return Err(Error::CircuitClosed);
+        if let RelayMsg::Truncated(t) = msg {
+            let reason = t.reason();
+            debug!(
+                "{}: Truncated from hop {}. Reason: {} [{}]",
+                self.unique_id,
+                hopnum,
+                reason.human_str(),
+                reason
+            );
+
+            return Ok(CellStatus::CleanShutdown);
         }
 
         trace!("{}: Received meta-cell {:?}", self.unique_id, msg);
@@ -793,7 +796,7 @@ impl Reactor {
                     ret
                 );
                 let _ = done.send(ret); // don't care if sender goes away
-                Ok(())
+                Ok(CellStatus::Continue)
             } else {
                 // Somebody wanted a message from a different hop!  Put this
                 // one back.
@@ -815,7 +818,7 @@ impl Reactor {
     }
 
     /// Handle a RELAY_SENDME cell on this circuit with stream ID 0.
-    fn handle_sendme(&mut self, hopnum: HopNum, msg: Sendme) -> Result<()> {
+    fn handle_sendme(&mut self, hopnum: HopNum, msg: Sendme) -> Result<CellStatus> {
         // No need to call "shutdown" on errors in this function;
         // it's called from the reactor task and errors will propagate there.
         let hop = self
@@ -823,13 +826,13 @@ impl Reactor {
             .ok_or_else(|| Error::CircProto(format!("Couldn't find {} hop", hopnum)))?;
 
         let auth: Option<[u8; 20]> = match msg.into_tag() {
-            Some(v) if v.len() == 20 => {
-                // XXXX ugly code.
-                let mut tag = [0_u8; 20];
-                (&mut tag).copy_from_slice(&v[..]);
-                Some(tag)
+            Some(v) => {
+                if let Ok(tag) = <[u8; 20]>::try_from(v) {
+                    Some(tag)
+                } else {
+                    return Err(Error::CircProto("malformed tag on circuit sendme".into()));
+                }
             }
-            Some(_) => return Err(Error::CircProto("malformed tag on circuit sendme".into())),
             None => {
                 if !hop.auth_sendme_optional {
                     return Err(Error::CircProto("missing tag on circuit sendme".into()));
@@ -838,10 +841,8 @@ impl Reactor {
                 }
             }
         };
-        match hop.sendwindow.put(auth) {
-            Some(_) => Ok(()),
-            None => Err(Error::CircProto("bad auth tag on circuit sendme".into())),
-        }
+        hop.sendwindow.put(auth)?;
+        Ok(CellStatus::Continue)
     }
 
     /// Send a message onto the circuit's channel (to be called with a `Context`)
@@ -906,6 +907,7 @@ impl Reactor {
         cell: RelayCell,
     ) -> Result<()> {
         let c_t_w = sendme::cell_counts_towards_windows(&cell);
+        let stream_id = cell.stream_id();
         let mut body: RelayCellBody = cell.encode(&mut rand::thread_rng())?.into();
         let tag = self.crypto_out.encrypt(&mut body, hop)?;
         let msg = chancell::msg::Relay::from_raw(body.into());
@@ -933,10 +935,44 @@ impl Reactor {
                 return Ok(());
             }
             hop.sendwindow.take(tag)?;
+            if !stream_id.is_zero() {
+                // We need to decrement the stream-level sendme window.
+                // Stream data cells should only be dequeued and fed into this function if
+                // the window is above zero, so we don't need to worry about enqueuing things.
+                if let Some(window) = hop.map.get_mut(stream_id).and_then(StreamEnt::send_window) {
+                    window.take(&())?;
+                } else {
+                    warn!(
+                        "{}: sending a relay cell for non-existent or non-open stream with ID {}!",
+                        self.unique_id, stream_id
+                    );
+                    return Err(Error::CircProto(format!(
+                        "tried to send a relay cell on non-open stream {}",
+                        stream_id
+                    )));
+                }
+            }
         }
         self.send_msg_direct(cx, msg)
     }
 
+    /// Try to install a given meta-cell handler to receive any unusual cells on
+    /// this circuit, along with a result channel to notify on completion.
+    fn set_meta_handler(
+        &mut self,
+        handler: Box<dyn MetaCellHandler>,
+        done: ReactorResultChannel<()>,
+    ) -> Result<()> {
+        if self.meta_handler.is_none() {
+            self.meta_handler = Some((handler, done));
+            Ok(())
+        } else {
+            Err(Error::InternalError(
+                "Tried to install a meta-cell handler before the old one was gone.".into(),
+            ))
+        }
+    }
+
     /// Handle a CtrlMsg other than Shutdown.
     fn handle_control(&mut self, cx: &mut Context<'_>, msg: CtrlMsg) -> Result<()> {
         trace!("{}: reactor received {:?}", self.unique_id, msg);
@@ -962,7 +998,7 @@ impl Reactor {
                     self,
                 ) {
                     Ok(e) => {
-                        self.meta_handler = Some((Box::new(e), done));
+                        self.set_meta_handler(Box::new(e), done)?;
                     }
                     Err(e) => {
                         let _ = done.send(Err(e));
@@ -1067,23 +1103,28 @@ impl Reactor {
     /// or rejected; a few get delivered to circuits.
     ///
     /// Return true if we should exit.
-    fn handle_cell(&mut self, cx: &mut Context<'_>, cell: ClientCircChanMsg) -> Result<bool> {
+    fn handle_cell(&mut self, cx: &mut Context<'_>, cell: ClientCircChanMsg) -> Result<CellStatus> {
         trace!("{}: handling cell: {:?}", self.unique_id, cell);
         use ClientCircChanMsg::*;
         match cell {
-            Relay(r) => {
-                self.handle_relay_cell(cx, r)?;
-                Ok(false)
-            }
-            Destroy(_) => {
+            Relay(r) => Ok(self.handle_relay_cell(cx, r)?),
+            Destroy(d) => {
+                let reason = d.reason();
+                debug!(
+                    "{}: Received DESTROY cell. Reason: {} [{}]",
+                    self.unique_id,
+                    reason.human_str(),
+                    reason
+                );
+
                 self.handle_destroy_cell()?;
-                Ok(true)
+                Ok(CellStatus::CleanShutdown)
             }
         }
     }
 
     /// React to a Relay or RelayEarly cell.
-    fn handle_relay_cell(&mut self, cx: &mut Context<'_>, cell: Relay) -> Result<()> {
+    fn handle_relay_cell(&mut self, cx: &mut Context<'_>, cell: Relay) -> Result<CellStatus> {
         let mut body = cell.into_relay_body().into();
 
         // Decrypt the cell. If it's recognized, then find the
@@ -1093,7 +1134,7 @@ impl Reactor {
         // copy it, but I don't see a way around it right now.
         let tag = {
             let mut tag_copy = [0_u8; 20];
-            // XXXX This could crash if the tag length changes.  We'll
+            // TODO(nickm): This could crash if the tag length changes.  We'll
             // have to refactor it then.
             (&mut tag_copy).copy_from_slice(tag);
             tag_copy
@@ -1153,6 +1194,7 @@ impl Reactor {
                 sink,
                 send_window,
                 dropped,
+                ref mut received_connected,
                 ..
             }) => {
                 // The stream for this message exists, and is open.
@@ -1161,10 +1203,15 @@ impl Reactor {
                     // We need to handle sendmes here, not in the stream's
                     // recv() method, or else we'd never notice them if the
                     // stream isn't reading.
-                    // FIXME(eta): I think ignoring the must_use return value here is okay, since
-                    //             the tag is () anyway? or something???
-                    let _ = send_window.put(Some(()));
-                    return Ok(());
+                    send_window.put(Some(()))?;
+                    return Ok(CellStatus::Continue);
+                }
+
+                if matches!(msg, RelayMsg::Connected(_)) {
+                    // Remember that we've received a Connected cell, and can't get another,
+                    // even if we become a HalfStream.  (This rule is enforced separately at
+                    // DataStreamReader.)
+                    *received_connected = true;
                 }
 
                 // Remember whether this was an end cell: if so we should
@@ -1210,7 +1257,7 @@ impl Reactor {
                 ));
             }
         }
-        Ok(())
+        Ok(CellStatus::Continue)
     }
 
     /// Helper: process a destroy cell.
