@@ -9,7 +9,7 @@ use tor_llcrypto::pk::curve25519::*;
 use tor_llcrypto::pk::rsa::RsaIdentity;
 use tor_llcrypto::util::rand_compat::RngCompatExt;
 
-use crypto_mac::{self, Mac, NewMac};
+use digest::Mac;
 use rand_core::{CryptoRng, RngCore};
 use zeroize::Zeroizing;
 
@@ -124,7 +124,7 @@ impl KeyGenerator for NtorHkdfKeyGenerator {
 }
 
 /// Alias for an HMAC output, used to validate correctness of a handshake.
-type Authcode = crypto_mac::Output<hmac::Hmac<d::Sha256>>;
+type Authcode = digest::CtOutput<hmac::Hmac<d::Sha256>>;
 
 /// Perform a client handshake, generating an onionskin and a state object
 fn client_handshake_ntor_v1<R>(
@@ -254,9 +254,10 @@ where
     R: RngCore + CryptoRng,
     T: AsRef<[u8]>,
 {
-    // XXXX we generate this key whether or not we are actually going to
-    // find our nodeid or keyid. Perhaps we should delay that till later.
-    // But if we do, we'll need to refactor a bit to keep our tests working.
+    // TODO(nickm): we generate this key whether or not we are
+    // actually going to find our nodeid or keyid. Perhaps we should
+    // delay that till later?  It shouldn't matter for most cases,
+    // though.
     let ephem = EphemeralSecret::new(rng.rng_compat());
     let ephem_pub = PublicKey::from(&ephem);
 
