@@ -326,7 +326,8 @@ fn client_handshake_ntor_v3_no_keygen(
 /// Trait for an object that handle and incoming client message and
 /// return a server's reply.
 ///
-// XXXX I wanted to use a closure here, but the lifetimes didn't work.
+// TODO(nickm): I wanted to use a closure here, but the lifetimes didn't work,
+// and I couldn't figure out why.
 pub(crate) trait MsgReply {
     /// Given a message received from a client, parse it and decide
     /// how (and whether) to reply.
@@ -408,7 +409,7 @@ fn server_handshake_ntor_v3_no_keygen<REPLY: MsgReply>(
     // unwrap_or_else() here, but that should be somewhat
     // hidden by the rest of the computation.
     okay &= ct::bool_to_choice(reply.is_some());
-    let reply = reply.unwrap_or_else(Vec::new);
+    let reply = reply.unwrap_or_default();
 
     // If we reach this point, we are actually replying, or pretending
     // that we're going to reply.
